@@ -23,28 +23,28 @@
 #
 #
 # Author: Ilya Baldin (ibaldin@renci.org)
-from typing import Dict, Any, List
+from typing import Tuple, Any, List
 
 import uuid
 
-from .model_element import ModelElement, ElementType
+from .model_element import ModelElement, ElementType, TopologyException
 
 from fim.graph.abc_property_graph import ABCPropertyGraph
 from fim.user.interface import Interface
 from fim.slivers.network_link import NetworkLinkSliver, LinkType
-from fim.slivers.switch_fabric import SFLayer
+from fim.slivers.network_service import NSLayer
 
 
 class Link(ModelElement):
     """
-    A link object in a topology. In addition to public methods the following calls
+    A link object in a topology connecting interfaces together.
+    In addition to public methods the following calls
     return various dictionaries or lists:
     link.interface_list - a list of interfaces attached to this link
     """
     def __init__(self, *, name: str, node_id: str = None, topo: Any,
                  etype: ElementType = ElementType.EXISTING,
                  interfaces: List[Interface] = None,
-                 layer: SFLayer = SFLayer.L2,
                  ltype: LinkType = None, technology: str = None, **kwargs):
         """
         Don't call this method yourself, call topology.add_link()
@@ -55,7 +55,6 @@ class Link(ModelElement):
         :param topo:
         :param etype: is this supposed to exist or new should be created
         :param interfaces: list of interface objects to connect
-        :param layer: layer of the link, defaults to L2
         :param ltype: link type if new
         :param technology: link technology
         :param kwargs: any additional properties
@@ -68,23 +67,23 @@ class Link(ModelElement):
             # node id myst be specified for new nodes in substrate topologies
             if str(topo.__class__) == "<class 'fim.user.topology.SubstrateTopology'>" and \
                     node_id is None:
-                raise RuntimeError("When adding new links to substrate topology nodes you must specify static Node ID")
+                raise TopologyException("When adding new links to substrate topology nodes you must specify static Node ID")
             if node_id is None:
                 node_id = str(uuid.uuid4())
             super().__init__(name=name, node_id=node_id, topo=topo)
             if ltype is None:
-                raise RuntimeError("When creating new links you must specify LinkType")
+                raise TopologyException("When creating new links you must specify LinkType")
             # FIXME isinstance
             if interfaces is None or len(interfaces) == 0 or (not isinstance(interfaces, tuple) and
                                                               not isinstance(interfaces, list)):
-                raise RuntimeError("When creating new links you must specify the list of interfaces to connect")
-
+                raise TopologyException("When creating new links you must specify the list of interfaces to connect.")
             self._interfaces = interfaces
             sliver = NetworkLinkSliver()
             sliver.node_id = self.node_id
             sliver.set_name(self.name)
             sliver.set_type(ltype)
-            sliver.set_layer(layer)
+            # set layer based on type
+            sliver.set_layer(NetworkLinkSliver.LinkConstraints[ltype].layer)
             sliver.set_technology(technology)
             sliver.set_properties(**kwargs)
 
@@ -99,9 +98,9 @@ class Link(ModelElement):
                 graph_model.find_node_by_name(node_name=name,
                                               label=ABCPropertyGraph.CLASS_Link)
             if existing_node_id != node_id:
-                raise RuntimeError(f'Link name {name} is not unique within the topology.')
+                raise TopologyException(f'Link name {name} is not unique within the topology.')
             # collect a list of interfaces it attaches to
-            interface_list = self.topo.graph_model.get_all_link_interfaces(link_id=self.node_id)
+            interface_list = self.topo.graph_model.get_all_ns_or_link_connection_points(link_id=self.node_id)
             name_id_tuples = list()
             # need to look up their names - a bit inefficient, need to think about this /ib
             for iff in interface_list:
@@ -109,9 +108,21 @@ class Link(ModelElement):
                 name_id_tuples.append((props[ABCPropertyGraph.PROP_NAME], iff))
             self._interfaces = [Interface(node_id=tup[1], topo=topo, name=tup[0]) for tup in name_id_tuples]
 
+    @property
+    def type(self):
+        return self.get_property('type') if self.__dict__.get('topo', None) is not None else None
+
+    @property
+    def technology(self):
+        return self.get_property('technology') if self.__dict__.get('topo', None) is not None else None
+
+    @property
+    def layer(self):
+        return self.get_property('layer') if self.__dict__.get('topo', None) is not None else None
+
     def get_property(self, pname: str) -> Any:
         """
-        Retrieve a interface property
+        Retrieve a link property
         :param pname:
         :return:
         """
@@ -121,11 +132,14 @@ class Link(ModelElement):
 
     def set_property(self, pname: str, pval: Any):
         """
-        Set a interface property
+        Set a link property or unset if pval is None
         :param pname:
         :param pval:
         :return:
         """
+        if pval is None:
+            self.unset_property(pname)
+            return
         link_sliver = NetworkLinkSliver()
         link_sliver.set_property(prop_name=pname, prop_val=pval)
         # write into the graph
@@ -145,7 +159,7 @@ class Link(ModelElement):
         self.topo.graph_model.update_node_properties(node_id=self.node_id, props=prop_dict)
 
     @staticmethod
-    def list_properties() -> List[str]:
+    def list_properties() -> Tuple[str]:
         return NetworkLinkSliver.list_properties()
 
     def __list_of_interfaces(self) -> List[Interface] or None:
@@ -157,19 +171,9 @@ class Link(ModelElement):
             return self._interfaces.copy()
         return None
 
-    def __getattr__(self, item):
-        """
-        Special handling for attributes like 'nodes' and 'links' -
-        which query into the model. They return dicts and list
-        containers. Modifying containers does not affect the underlying
-        graph mode, but modifying elements of lists or values of dicts does.
-        :param item:
-        :return:
-        """
-
-        if item == 'interface_list':
-            return self.__list_of_interfaces()
-        raise RuntimeError(f'Attribute {item} not available')
+    @property
+    def interface_list(self):
+        return self.__list_of_interfaces()
 
     def __repr__(self):
         _, node_properties = self.topo.graph_model.get_node_properties(node_id=self.node_id)
